// ==============================================================
//
//  Copyright (c) 1999-2004 by Alex Vinokur.
//
//  For conditions of distribution and use, see
//  copyright notice in version.h
//
// ==============================================================


// ##############################################################
// ==================================
//  n-ary Huffman Template Algorithm
// ==================================
//
//  FILE : raw.h
//
//  AUTHOR : Alex Vinokur
//
//  DESCRIPTION :
//
//
//         Treatment of raw frequencies (Definitions)
//         ------------------------------------------
//
// ##############################################################

#ifndef RAW_H
#define RAW_H


// ===================
#include "methods.h"
// ===================


// ===================
#ifndef ARITY
#define	ARITY 	2
#endif

#define	END_OF_TEXT_FILE_ARGV	"---"

// ----------------------
int usage_help (int argc, char **argv);



// ----------- class Aux -----------
class AuxPairSymbolCode
{
template <typename W1, unsigned int A1> friend class MixedHuffmanTree;
friend class lessAuxPairSymbolCode;  
friend ostream& operator<< (ostream& o, const AuxPairSymbolCode& inst_i);  

  private :
   string	ext_symbol_;
   vector<CODE>	code_path_;

  public :
   AuxPairSymbolCode (const string& ext_symbol_i, const vector<CODE>& code_path_i) : 
	ext_symbol_ (ext_symbol_i),	
	code_path_ (code_path_i)
	{}
   ~AuxPairSymbolCode () {}

};

// ----------- class lessAuxPairSymbolCode -----------
class lessAuxPairSymbolCode
{
  public:
    bool operator()(
	const AuxPairSymbolCode& inst1_i,
	const AuxPairSymbolCode& inst2_i
	);
};



// ----------- template class LoadedHuffmanTree -----------
template <typename WEIGHT = freq_t, unsigned int ARY = 2>
class MixedHuffmanTree : public LoadedHuffmanTree<string, WEIGHT, ARY>
{

  private :
    // -------------------------------
    const vector<string>	text_file_names_;
    const size_t		max_ext_freq_stage_;
    const vector<string>	msg_file_name_;
    vector<string>		source_strings_;

    vector<map<string, WEIGHT> >	ext_frequencies_;
    vector<bool>			recomputed_frequencies_;
    vector<freq_t>			weighted_code_sizes_sum_;
    vector<AuxPairSymbolCode>		vect_symbols_codes_;
    mutable vector<ullong>		accum_sizes_;

    // -------------------------------
    static size_t	get_pattern_frequencies_s (const string& str_i, const string& pattern_i);
    static vector<string>	remove_substr_s (const string& str_i, const string& substr_i);
    static void		compute_frequencies_s (map<string, WEIGHT>& map_io, const string& str_i);
    static void		add_frequencies_s (const map<string, WEIGHT>& source_map_i, map<string, WEIGHT>& result_map_io);
    static void		show_frequencies_s (
				const map<string, WEIGHT>&	map_i, 
				const string&			cur_source_str_i,
				size_t				pattern_size_i,
				const string&			info_type_i,
				const string&			msg_i  = string ()
				);
    static void		init_frequencies_s (
	   			map<string, WEIGHT>&	map_o,
				size_t			pattern_size_i, 
				const string&		str_i
				);

    static typename map<string, WEIGHT>::const_iterator	get_max_frequency_s (const map<string, WEIGHT>& map_i);

    void	show_frequencies () const;
    void	compute_extended_frequencies (size_t pattern_size_i);
    void	compute_extended_frequencies ();
    void	recompute_frequencies (size_t alphabet_level_i);
    void	show_all_frequencies (const string& msg_i = string()) const;
    void	build_source_strings ();
    bool	get1_msg_head_and_tail (
			size_t		alphabet_level_i,
			string&		head_o,
			string&		tail_io
			) const;
    bool	get4_msg_head_and_tail (
			size_t		alphabet_level_i,
			string&		head_o,
			string&		tail_io
			) const;
    string		build_string_from_file (const string& msg_file_name_i) const;
    vector<string>	build1_vect_msg (const string& str_msg_i, size_t alphabet_level_i) const;
    vector<string>	build3i_vect_msg (const string& str_msg_i) const;
    vector<string>	build3_vect_msg (const string& str_msg_i, size_t alphabet_level_i = 0) const;
    vector<string>	build4_vect_msg (const string& str_msg_i, size_t alphabet_level_i) const;
    vector<string>	build5_vect_msg (const string& str_msg_i, size_t alphabet_level_i = 0) const;
    vector<Cell<string, WEIGHT> > fill_data_vector (size_t alphabet_level_i) const;
    bool	encode_decode_message (
			bool		source_text_flag_i,
			const string&	msg_file_name_i,
			const string&	msg_str_i,
			size_t		alphabet_level_i	
			) const;
    void	create_mixed_tree (size_t alphabet_level_i);
    void	create_vect_symbols_codes (size_t alphabet_level_i);
    double	get_superior_compression_factor (size_t alphabet1_i, size_t alphabet2_i) const;
    double	get_actual_compression_factor (size_t alphabet1_i, size_t alphabet2_i) const;

  public :
    MixedHuffmanTree ();

    MixedHuffmanTree (
	const vector<string>&	text_file_names_i, 
	const vector<string>&	msg_file_name_i,
        size_t			max_ext_freq_stage_i = 2
	);

   ~MixedHuffmanTree ();
};

#endif	// RAW_H

