// ==============================================================
//
//  Copyright (c) 1999-2004 by Alex Vinokur.
//
//  For conditions of distribution and use, see
//  copyright notice in version.h
//
// ==============================================================



// ##############################################################
// ==================================
//  n-ary Huffman Template Algorithm
// ==================================
//
//  FILE : service.cpp
//
//  AUTHOR : Alex Vinokur
//
//  DESCRIPTION :
//
//         Implementation of the following auxiliary functions : 
//         -----------------------------------------------------
//         - void               show_file (...)
//         -----------------------------------------------------
//
// ##############################################################


// ===========================
#include "service.h"
// ===========================


// #####################################################
template <>
string gstr_symbol (const char& ch_i)
{
const int left_bound = 16;
const int right_bound = 17;
  if (isgraph(ch_i)) return to_str (ch_i);
  return (char (left_bound) + string ("\\" + to_str (int (static_cast<unsigned char> (ch_i)))) + char (right_bound));
}

// #####################################################
template <>
string gstr_symbol (const string& str_i)
{
string ret_str;
  for (size_t i = 0; i < str_i.size(); i++) ret_str += gstr_symbol (str_i[i]);
  return ret_str;
}


// #####################################################
void   show_file (
	const string& file_type_i, 
	const string& file_name_i, 
	const string& delimiter_i
	)
{
string		ret_string;
string		file_line;
ifstream	fin (file_name_i.c_str());

	if (!fin)
	{
	       	ret_string = "Unable to open file " + file_name_i;	
		ERROR_MSG(ret_string);
		cout << ret_string << endl;
		return;
	}
	assert (fin);


	cout << endl;
	cout << "\t-----------------------" << endl;
	cout << "\t--- " << file_type_i << " file : " << file_name_i << endl;
	cout << "\t-----------------------" << endl;

	for (int i = 1; getline (fin, file_line); i++)
	{
		vector<string> file_vline;
		istringstream isstr (file_line.c_str ());

		copy(
			istream_iterator<string>(isstr),
			istream_iterator<string> (),
			back_inserter (file_vline)
			);

		cout << "\t  Line#" << i << " ---> ";

		copy (
			file_vline.begin (),
			file_vline.end (),
			ostream_iterator<string> (cout, delimiter_i.c_str())
			);
		cout << endl;

	} // for

		cout << "\t-----------------------" << endl;

} // string   show_file

// #####################################################
bool get_file_size(
	const string&	filename_i,
	ullong&		filesize_o
	)
{
ifstream fin (filename_i.c_str());

  if (!fin) 
  {
    ERROR_MSG ("Cannot open file <"
              << filename_i
              << "> to get its filesize"
              << endl
              << ERROR_SHIFT
              );
    return false;
  }

ullong	start_pos, end_pos;
  start_pos = fin.tellg();
  fin.seekg(0, ios::end); 
  end_pos = fin.tellg();

  filesize_o = end_pos - start_pos;
  assert (filesize_o < MAX_FILESIZE_VALUE);

  return true;

} // get_file_size



// #######################################################
// ##### PART : CLASSES ##################################
// #######################################################
// =============================
// Constructor-0
ClassLogo::ClassLogo ()
{
	show_logo ("START");
}

// =============================
// Destructor
ClassLogo::~ClassLogo ()
{
	cout << endl;
	cout << endl;
	cout << endl;
	show_logo ("FINISH");
}

// =============================
void	ClassLogo::show_logo (const string& msg_i) const
{
time_t timer;
	timer	= time(NULL);

const string pref1 ("\t#");
const string pref2 (pref1 + " ");

vector<string>	product_info;
vector<string>	author_info;

  product_info.push_back (product_name);
  product_info.push_back (string (2, ' ') + string (product_url));
  product_info.push_back (string (2, ' ') + string (sw_version));

  author_info.push_back (string (2, ' ') + author_name);
  author_info.push_back (string (2, ' ') + author_url);
  author_info.push_back (string (2, ' ') + author_email);

vector<vector<string> >	system_info;

#if (__DJGPP__)
  system_info.push_back(vector<string>());
  system_info.rbegin()->push_back (pref2);
  system_info.rbegin()->push_back ("DJGPP ");
  system_info.rbegin()->push_back (to_str(__DJGPP__));
#ifdef __DJGPP_MINOR__ 
  system_info.rbegin()->push_back (".");
  system_info.rbegin()->push_back (to_str(__DJGPP_MINOR__));
#endif
#endif  

#if (__MINGW32__ || __MINGW__ )
  system_info.push_back(vector<string>());
  system_info.rbegin()->push_back (pref2);
  system_info.rbegin()->push_back ("MINGW");
#endif  

#if (__CYGWIN32__ || __CYGWIN__)
  system_info.push_back(vector<string>());
  system_info.rbegin()->push_back (pref2);
  system_info.rbegin()->push_back ("CYGWIN");
#endif  

#ifdef __GNUC__ 
  system_info.push_back(vector<string>());
  system_info.rbegin()->push_back (pref2);
  system_info.rbegin()->push_back ("GNU gcc version ");
  system_info.rbegin()->push_back (to_str(__GNUC__));
#ifdef __GNUC_MINOR__  
  system_info.rbegin()->push_back (".");
  system_info.rbegin()->push_back (to_str(__GNUC_MINOR__));

#ifdef __GNUC_PATCHLEVEL__   
#if (__GNUC_PATCHLEVEL__)
  system_info.rbegin()->push_back (".");
  system_info.rbegin()->push_back (to_str(__GNUC_PATCHLEVEL__));
#endif
#endif

#endif
#endif  


size_t the_width = 0;
  for (size_t i = 0; i < product_info.size(); i++)
  {
    the_width = MAX_VALUE (the_width, product_info[i].size());
  }
  for (size_t i = 0; i < author_info.size(); i++)
  {
    the_width = MAX_VALUE (the_width, author_info[i].size());
  }

  cout << endl;
  cout << pref1 << string (the_width + 1, '=') << endl;
  for (size_t i = 0; i < product_info.size(); i++)
  {
    cout << pref2 << product_info[i] << endl;
  }
  cout << pref1 << string (the_width + 1, '.') << endl;
  for (size_t i = 0; i < author_info.size(); i++)
  {
    cout << pref2 << author_info[i] << endl;
  }


  if (!system_info.empty())
  {
    cout << pref1 << string (the_width + 1, '-') << endl;
  }
  for (size_t i = 0; i < system_info.size(); i++)
  {
    copy (system_info[i].begin(), system_info[i].end(), ostream_iterator<string> (cout, ""));
    cout << endl;
  }
  cout << pref1 << string (the_width + 1, '-') << endl;

  cout << pref2 << msg_i << endl;
  cout << pref2 << asctime (localtime(&timer));
  cout << pref1 << string (the_width + 1, '=') << endl;
  cout << endl;

}

// #######################################################
ClassLogo	logo;


// #######################################################
// ################ END OF FILE ##########################
// #######################################################

